/*
 *  mvPCIe Driver - DMA implementation
 *  Copyright (c) 2019-2020 by MATRIX VISION GmbH <info@matrix-vision.de>
 *  Author: grayh (based on previous work by mrakk and others)
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 *
 */

#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/mm.h>
#include <linux/pagemap.h>
#include <linux/version.h>

#include "mvdma.h"

#if LINUX_VERSION_CODE < KERNEL_VERSION(5, 0, 0)
static void mvdma_dma_callback(void *data, struct timeval *tv, u32 img_size,
                   u32 img_id, u32 status_NOT_USED)
#else
static void mvdma_dma_callback(void *data, struct timespec64 *ts, u32 img_size,
                   u32 img_id, u32 status_NOT_USED)
#endif
{
    TmvUI_umem *um = (TmvUI_umem *) data;

    um->dma_state = DMA_DONE;
#if LINUX_VERSION_CODE < KERNEL_VERSION(5, 0, 0)
    um->tv.tv_sec = tv->tv_sec;
    um->tv.tv_usec = tv->tv_usec;
#else
    um->tv.tv_sec = ts->tv_sec;
    um->tv.tv_usec = ts->tv_nsec / NSEC_PER_USEC;
#endif
    um->img_size = img_size;
    um->img_id = img_id;
    wake_up(&um->wq);
}

ssize_t mvdma_dma_to_user(TmvUI_umem * um)
{
    TmvUI_userhandle *uh = um->uh;
    struct mvdma_device *pdma = uh->pdma;

    dprintk_dma(MVDMA_DEBUG_QUEUE,
            "sl @ mem %d (%d entries), expect img_size 0x%x\n", um->nr,
            um->page_cnt, um->img_size);
    um->dma_state = DMA_RUNNING;

    return mvdma_request(uh->pdma, um->sl, um->page_cnt, mvdma_dma_callback,
                 um);
    return 0;
}
